--
-- smokeParticles
--
-- @author:        Xentro (www.fs-uk.com)(Marcus@Xentro.se)
-- @version:    v1.0
-- @date:       2013-11-29
-- 
--[[
<smokeParticles fillTypes="FILLTYPE FILLTYPE" cooldown="MINUTES" moveableNode="INDEX" min="X Y Z" max="X Y Z">
    <particle node="INDEX" position="0 0 0" rotation="0 0 0" file="FILE" />
</smokeParticles>
]]--

smokeParticles = {};

function smokeParticles.prerequisitesPresent(specializations)
    if not SpecializationUtil.hasSpecialization(Fillable, specializations) then print("Warning: Specialization smokeParticles needs the specialization Fillable."); end;
    
    return SpecializationUtil.hasSpecialization(Fillable, specializations);
end;

function smokeParticles:load(savegame)
    self.setParticles = SpecializationUtil.callSpecializationsFunction("setParticles");
    self.getMovePosition = smokeParticles.getMovePosition;
    self.setFillLevel = Utils.prependedFunction(self.setFillLevel, smokeParticles.setFillLevel);
    
    self.smokeP = {};
    self.smokeP.isEmitting = false;
    self.smokeP.lastFillLevel = 0;
    self.smokeP.curTime = 0;
    self.smokeP.lastTime = 0;
    self.smokeP.cooldown = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.smokeParticles#cooldown"), 10);
    self.smokeP.bcts = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.smokeParticles#baseCooldownOnTimeScale"), true);
    
    local moveableNode = getXMLString(xmlFile, "vehicle.smokeParticles#moveableNode");
    if moveableNode ~= nil then
        self.smokeP.moveable = {};
        self.smokeP.moveable.node = Utils.indexToObject(self.components, moveableNode);
        
        self.smokeP.moveable.min = Utils.getVectorNFromString(getXMLString(self.xmlFile, "vehicle.smokeParticles#min"), 3);
        self.smokeP.moveable.max = Utils.getVectorNFromString(getXMLString(self.xmlFile, "vehicle.smokeParticles#max"), 3);
        
        if self.smokeP.moveable.min == nil or self.smokeP.moveable.max == nil then
            print("smokeParticles - Error: smokeParticles min or max does not have all XYZ value set.");
            self.smokeP.moveable.node = nil;
        end;
        
        setTranslation(self.smokeP.moveable.node, unpack(self.smokeP.moveable.min));
    end;
    
    self.smokeP.allowedFillTypes = {};
    local fillTypes = getXMLString(self.xmlFile, "vehicle.smokeParticles#fillTypes");
    if fillTypes ~= nil then
        local types = Utils.splitString(" ", fillTypes);
        
        for k, v in pairs(types) do
            local fillTypeInt = Fillable.fillTypeNameToInt[v];
            if fillTypeInt ~= nil then
                self.smokeP.allowedFillTypes[fillTypeInt] = true;
            end;
        end;
    end;
    
    if self.isClient then
        self.smokeP.nodes = {};
        local i = 0;
        while true do 
            local key = string.format("vehicle.smokeParticles.particle(%d)", i);
            if not hasXMLProperty(xmlFile, key) then break;    end;
            local currentPS = {};
            local particleNode = Utils.loadParticleSystem(self.xmlFile, currentPS, key, self.components, false, "particleSystems/poopSteam.i3d", self.baseDirectory);

            table.insert(self.smokeP.nodes, currentPS);
            i = i + 1;
        end;
    end;
end;

function smokeParticles:delete()
    if self.isClient then
        for _, particleSystem in pairs(self.smokeP.nodes) do
            Utils.deleteParticleSystem(particleSystem);
        end;
    end;
end;

function smokeParticles:readStream(streamId, connection)
end;

function smokeParticles:writeStream(streamId, connection)
end;

function smokeParticles:mouseEvent(posX, posY, isDown, isUp, button)
end;

function smokeParticles:keyEvent(unicode, sym, modifier, isDown)
end;

function smokeParticles:update(dt)
end;

function smokeParticles:updateTick(dt)
    if self.smokeP.curTime > 0 and self.fillLevel > 0 then
        if not self.smokeP.isEmitting then
            self:setParticles(true);
        else
            local dtMinutes = dt / 60000;
            
            if self.smokeP.bcts then
                dtMinutes = dtMinutes * g_currentMission.environment.timeScale;
            end;
            
            self.smokeP.curTime = self.smokeP.curTime - dtMinutes;
            
            if self.smokeP.lastTime ~= self.smokeP.curTime or (self.tipState ~= nil and self.tipState == Trailer.TIPSTATE_OPENING) then
                local scale = math.max(self.smokeP.curTime, 0) / self.smokeP.cooldown;
                if self.isClient then
                    for _, particleSystem in pairs(self.smokeP.nodes) do
                        Utils.setEmitCountScale(particleSystem, scale);
                    end;
                end;
                
                if self.smokeP.moveable ~= nil then
                    local trans = {};
                    for i = 1, 3 do
                        trans[i] = self:getMovePosition(self.smokeP.moveable.min[i], self.smokeP.moveable.max[i]);
                    end;
                    setTranslation(self.smokeP.moveable.node, unpack(trans));
                end;
                
                self.smokeP.lastTime = self.smokeP.curTime;
            end;
        end;
    else
        if self.smokeP.isEmitting then
            self:setParticles(false);
        end;
    end;
end;

function smokeParticles:draw()
end;

function smokeParticles:setParticles(state)
    if self.isClient then
        for _, particleSystem in pairs(self.smokeP.nodes) do
            Utils.setEmittingState(particleSystem, state);
        end;
    end;
    
    self.smokeP.isEmitting = state;
    
    if not state then
        self.smokeP.curTime = 0;
    end;
end;

function smokeParticles:setFillLevel(fillLevel, fillType, force)
    if self.smokeP.allowedFillTypes[fillType] ~= nil and self.smokeP.allowedFillTypes[fillType] then
        if fillLevel > self.smokeP.lastFillLevel then
            self.smokeP.curTime = self.smokeP.cooldown;
        end;
        
        self.smokeP.lastFillLevel = fillLevel;
    end;
end;

function smokeParticles:getMovePosition(minTrans, maxTrans)
    return minTrans + (maxTrans - minTrans) * Utils.clamp(self.fillLevel, 0, self.capacity) / (self.capacity);
end;